package de.hft.stuttgart.citydoctor2.utils;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.zip.CityGmlZipArchive;
import de.hft.stuttgart.citydoctor2.zip.CityGmlZipEntry;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class ArchivePacker {

    private static final Logger logger = LogManager.getLogger(ArchivePacker.class);

    private ArchivePacker(){}

    public static void packArchive(String targetPath, CityGmlZipArchive archive){
        Path outputPath = Path.of(targetPath);
        outputPath = outputPath.getParent().resolve(
                FilenameUtils.removeExtension(outputPath.getFileName().toString()) + ".zip");
        Path tmpDir = null;
        try{
            tmpDir = Files.createTempDirectory("zipTmp");
            tmpDir.toFile().deleteOnExit();
            for (CityGmlZipEntry entry : archive.getEntries()) {
                if (entry.getErrorType() != null || !entry.isDecompressed()) {
                    continue;
                }
                CityDoctorModel model = entry.getModel();
                Path filePath = tmpDir.resolve(entry.getEntrySubPath());
                Files.createDirectories(filePath.getParent());
                model.saveAs(filePath.toString(), model.isValidated());
            }
            zipFolder(tmpDir.toFile(), outputPath.toFile());

        }catch (Exception e){
            logger.error(e);
        } finally {
            if (tmpDir != null) {
                try {
                    FileUtils.deleteDirectory(tmpDir.toFile());
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
    }


    public static void zipFolder(File folder, File zipFile) throws IOException {
        zipFolder(folder, new FileOutputStream(zipFile));
    }

    public static void zipFolder(File folder, OutputStream outputStream) throws IOException {
        try (ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
            processFolder(folder, zipOutputStream, folder.getPath().length() + 1);
        }
    }

    private static void processFolder(File folder, ZipOutputStream zipOutputStream, int prefixLength)
            throws IOException {
        for (File file : folder.listFiles()) {
            if (file.isFile()) {
                ZipEntry zipEntry = new ZipEntry(file.getPath().substring(prefixLength));
                zipOutputStream.putNextEntry(zipEntry);
                try (FileInputStream inputStream = new FileInputStream(file)) {
                    IOUtils.copy(inputStream, zipOutputStream);
                }
                zipOutputStream.closeEntry();
            } else if (file.isDirectory()) {
                processFolder(file, zipOutputStream, prefixLength);
            }
        }
    }

    public static void packAndDeleteDirectory(String directoryPath) throws IOException {
        Path sourcePath = Path.of(directoryPath);
        if (!Files.exists(sourcePath)) {
            throw new IllegalStateException("Directory " + directoryPath + " does not exist");
        }

        if (!Files.isDirectory(sourcePath)) {
            throw new IllegalStateException("Path " + directoryPath + " is not a directory");
        }
        Path outputPath = sourcePath.getParent().resolve(sourcePath.getFileName() + ".zip");
        zipFolder(sourcePath.toFile(), outputPath.toFile());

        FileUtils.deleteDirectory(sourcePath.toFile());
    }

}
