package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.parser.ProgressListener;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.io.Serial;
import java.io.Serializable;
import java.nio.file.Path;
import java.util.zip.ZipEntry;

public class CityGmlZipEntry implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);
    @Serial
    private static final long serialVersionUID = -5732913269959043262L;

    private final Path filePath;
    private final String fileName;
    private final String displayName;
    private transient CityDoctorModel model;
    private CityGmlZipArchive parentArchive;
    private boolean decompressed;
    private long fileSize = -1L;
    private static final long MB = 1024 * 1024L;
    private ZipEntryErrorType errorType = null;

    public static CityGmlZipEntry of(ZipEntry entry, CityGmlZipArchive parentArchive, ParserConfiguration config) {
        CityGmlZipEntry ze = CityGmlZipEntry.register(entry, parentArchive);
        ze.loadEntry(config);
        return ze;
    }

    public void loadEntry(ParserConfiguration config) {
        loadEntry(config, null);
    }

    public void loadEntry(ParserConfiguration config, ProgressListener l) {
        if (decompressed) {
            return;
        }
        if (errorType != null) {
            logger.warn("Tried loading erroneous CityGmlZipEntry");
            return;
        }
        try {
            this.model = CityGmlParser.parseCityGmlZipEntry(this, config, l);
            this.decompressed = true;
        } catch (CityGmlParseException | InvalidGmlFileException e) {
            logger.error(e);
            this.errorType = ZipEntryErrorType.INVALID_CITY_GML_FILE;
        } catch (IOException e) {
            logger.error(e);
            this.errorType = ZipEntryErrorType.IO_ERROR;
        }
    }

    public static CityGmlZipEntry register(ZipEntry entry, CityGmlZipArchive parentArchive) {
        CityGmlZipEntry cgzEntry = new CityGmlZipEntry(entry, parentArchive, false);
        try {
            if (!cgzEntry.entrySizeWithinMemoryLimits()) {
                cgzEntry.errorType = ZipEntryErrorType.EXCESSIVE_FILESIZE;
            }
        } catch (IOException e) {
            logger.error(e);
            cgzEntry.errorType = ZipEntryErrorType.IO_ERROR;
        }
        return cgzEntry;
    }

    private boolean entrySizeWithinMemoryLimits() throws IOException {
        long memoryLimit = (long) Math.ceil(((double) Runtime.getRuntime().maxMemory() / MB) * 0.9);
        if (fileSize == -1L) {
            try (CityGmlZipEntryFile entryFile = new CityGmlZipEntryFile(this)) {
                long filesize = entryFile.getFileSize();
                if (filesize != -1) {
                    this.fileSize = filesize;
                } else {
                    return false;
                }
            } catch (Exception e) {
                throw new IOException(e);
            }
        }
        return memoryLimit > (fileSize / MB);
    }

    protected CityGmlZipEntry(ZipEntry entry, CityGmlZipArchive parentArchive, boolean decompressed) {
        this.fileName = entry.getName();
        this.filePath = Path.of(entry.getName());
        this.displayName = filePath.getFileName().toString();
        if (entry.getSize() != -1) {
            this.fileSize = entry.getSize();
        }
        this.model = null;
        this.decompressed = decompressed;
        this.parentArchive = parentArchive;
    }

    public void setArchive(CityGmlZipArchive archive) {
        parentArchive = archive;
    }

    public CityGmlZipArchive getArchive() {
        return parentArchive;
    }

    /**
     * Gets this entry's sub-path in the archive.
     *
     * @return Relative sub-path from the archive root directory.
     */
    public String getEntrySubPath() {
        return fileName;
    }

    public String getDisplayName() {
        return displayName;
    }

    public ZipEntryErrorType getErrorType() {
        return errorType;
    }

    public CityDoctorModel getModel() {
        return model;
    }

    public void setFileSize(long size) {
        fileSize = size;
    }

    public Path resolveLibraryObjectPath(String libraryObjectPath) {
        return filePath.getParent().resolve(libraryObjectPath);
    }

    public boolean isDecompressed() {
        return decompressed;
    }

    /**
     * Returns the estimated uncompressed filesize of this entry.
     *
     * @return filesize in bytes, -1 if filesize unknown
     */
    public long getFileSize() {
        return fileSize;
    }
}
